clear all;
close all;
clc;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load ws_efficiency_powerCast
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%maximum number of coherence times
N_max = 1;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%carrier frequency and propagation wavelength
F_car = 868*10^6;
lambda_c =  (3*10^8)/ F_car;
%reference distance
d_ref = 1;
%transmit power
P_tx_all_dBm =  [25 : 2: 35];
P_tx_all = 10.^(P_tx_all_dBm/10);  %mWatt
%path loss exponent
nu = 2.1;
%nakagami parameter
M_nakagami = 5;
%Time duration of packet in seconds
T_pack = 0.05; %secs
%distance
d_all = [3] ; %meters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%% Proposed methododology
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%sensitivity and saaturation of powerCast module  
sens_rect = 10^(sens_powerCast_dBm/10);
satur_rect = 10^(satur_powerCast_dBm/10);
% P_in axis for approximation in dB
P_in_test_dB = sens_powerCast_dBm : 0.1 : satur_powerCast_dBm ;

%P_in axis for approximation in mWatt
b_all = 10.^(P_in_test_dB/10);
%P_out function evaluated at b_all
v_all =  (P_harv_function( P_in_test_dB, sens_powerCast_dBm, satur_powerCast_dBm, w_opt ));
plot(b_all, v_all)
%slopes
l_all = (v_all(2:end) - v_all(1:end-1) ) ./ (b_all(2:end) - b_all(1:end-1) ) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%monte carlo iter
MC_MaxIter = 200000;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%expected harvested energies
EHE_exact_MC = zeros(length(d_all), length(P_tx_all));
EHE_proposed_analysis = zeros(length(d_all), length(P_tx_all));

EHE_Lin_analysis0 = zeros(length(d_all), length(P_tx_all));
EHE_Lin_analysis1 = zeros(length(d_all), length(P_tx_all));
EHE_Lin_analysis2 = zeros(length(d_all), length(P_tx_all));
% EHE_Lin_MC = zeros(length(d_all), N_max);
EHE_Con_Lin_analysis0 = zeros(length(d_all), length(P_tx_all));
EHE_Con_Lin_analysis1 = zeros(length(d_all), length(P_tx_all));
EHE_Con_Lin_analysis2 = zeros(length(d_all), length(P_tx_all));
% EHE_Con_Lin_MC = zeros(length(d_all), N_max);
EHE_Con_Lin_Con_analysis0 = zeros(length(d_all), length(P_tx_all));
EHE_Con_Lin_Con_analysis1 = zeros(length(d_all), length(P_tx_all));
EHE_Con_Lin_Con_analysis2 = zeros(length(d_all), length(P_tx_all));
% EHE_Con_Lin_Con_MC = zeros(length(d_all), N_max);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%etas for the baseline models
eta_L0 = 0.0001;
eta_CL0 = 0.05;
eta_CLC0 = 0.05;
eta_L1 = 0.005;
eta_CL1 = 0.1;
eta_CLC1 = 0.1;
eta_L2 = 0.02;
eta_CL2 = 0.2;
eta_CLC2 = 0.2;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


for P_tx_iter = 1 : length(P_tx_all)
    
    %current transmit power
    P_tx = P_tx_all(P_tx_iter)
    
    for  d_iter = 1 : length(d_all)
        
        %current distance
        d = d_all(d_iter);
        
        %curent average received power
        P_rec_avg = P_tx * (lambda_c/(4*pi*d_ref))^2 * (d_ref./d).^nu;
        
        %% proposed model
        %area within (b_{i-1}, b_{i}) accross all i
        CDF_diff = gammainc(b_all(2:end)*M_nakagami./(P_rec_avg), M_nakagami) -  gammainc(b_all(1:end-1)*M_nakagami./(P_rec_avg),M_nakagami) ;
        
        %M coeficients for mean value
        mean_coeffs =     l_all .* (P_rec_avg) .*(  gammainc(b_all(1:end-1)*M_nakagami./(P_rec_avg), M_nakagami+1,'upper') -...
            gammainc(b_all(2:end)*M_nakagami./(P_rec_avg), M_nakagami+1,'upper')   ) + (v_all(1:end-1) - l_all.* b_all(1:end-1)).*CDF_diff + ...
            v_all(end)*(1-gammainc(b_all(end)*M_nakagami./(P_rec_avg),M_nakagami));
        
        
        %number of coherence blocks
        n = N_max;
        
        
        
        %analysis for baseline models
        %%%%%%%%%%%%%%%%%%%%%%%%
        EHE_Lin_analysis0(d_iter,P_tx_iter) = n*P_rec_avg*T_pack*eta_L0;
        EHE_Con_Lin_analysis0(d_iter,P_tx_iter) = n*T_pack*eta_CL0*(P_rec_avg*gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper') ...
            - sens_rect *  gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper'));
        EHE_Con_Lin_Con_analysis0(d_iter,P_tx_iter) =  n*T_pack*eta_CLC0*( ( P_rec_avg*(gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper')...
            - (gammainc(satur_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper') ) ) )  + ...
            satur_rect *  gammainc(satur_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper') - ...
            sens_rect *  gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper') ) ;
        
        
        EHE_Lin_analysis1(d_iter,P_tx_iter) = n*P_rec_avg*T_pack*eta_L1;
        EHE_Con_Lin_analysis1(d_iter,P_tx_iter) = n*T_pack*eta_CL1*(P_rec_avg*gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper') ...
            - sens_rect *  gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper'));
        EHE_Con_Lin_Con_analysis1(d_iter,P_tx_iter) =  n*T_pack*eta_CLC1*( ( P_rec_avg*(gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper')...
            - (gammainc(satur_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper') ) ) )  + ...
            satur_rect *  gammainc(satur_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper') - ...
            sens_rect *  gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper') ) ;
        
        EHE_Lin_analysis2(d_iter,P_tx_iter) = n*P_rec_avg*T_pack*eta_L2;
        EHE_Con_Lin_analysis2(d_iter,P_tx_iter) = n*T_pack*eta_CL2*(P_rec_avg*gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper') ...
            - sens_rect *  gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper'));
        EHE_Con_Lin_Con_analysis2(d_iter,P_tx_iter) =  n*T_pack*eta_CLC2*( ( P_rec_avg*(gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper')...
            - (gammainc(satur_rect*M_nakagami./(P_rec_avg) , M_nakagami+1 ,'upper') ) ) )  + ...
            satur_rect *  gammainc(satur_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper') - ...
            sens_rect *  gammainc(sens_rect*M_nakagami./(P_rec_avg) , M_nakagami ,'upper') ) ;
        %%%%%%%%%%%%%%%%%%%%%%%%
        
        
        
        %%%%%%%%%%%%%%%%%%%%%%%%
        %analysis for the proposed model
        EHE_proposed_analysis(d_iter,P_tx_iter) = n*T_pack*sum(mean_coeffs);
        %%%%%%%%%%%%%%%%%%%%%%%%
        
        %parameter to generate gamma random variables
        gamma_param_1 = M_nakagami *  ones(1,MC_MaxIter);
        gamma_param_2 = P_rec_avg./gamma_param_1;
        
        %generate gamma random variables with the specific parameters
        P_in =  gamrnd(gamma_param_1 ,gamma_param_2  );
        
        
        %harvested power
        P_harv = P_harv_function(10*log10(P_in), sens_powerCast_dBm, satur_powerCast_dBm, w_opt );
        
        %calculate the exact values for expected harvesting energy
        EHE_exact_MC(d_iter, P_tx_iter) = n*T_pack*mean(P_harv);
        
    end
end


%%

E_all1 = [ EHE_proposed_analysis(1,:);...
    EHE_exact_MC(1,:); ... EAE_linear_approx3(:,1)';
    %      EHE_Lin_analysis0(:,1)';...
    %        EHE_Con_Lin_analysis0(:,1)'; ...
    %       EHE_Con_Lin_Con_analysis0(:,1)' ;
    EHE_Lin_analysis2(1,:);...
    EHE_Con_Lin_analysis2(1,:); ...
    EHE_Con_Lin_Con_analysis2(1,:) ;...
    EHE_Lin_analysis1(1,:);...
    EHE_Con_Lin_analysis1(1,:); ...
    EHE_Con_Lin_Con_analysis1(1,:)];
% semilogy(d_all, E_all1 )



% semilogy(d_all, E_all2 )
% hold on
% % Create figure
figure1 = figure;
%
% Create axes
axes1 = axes('Parent',figure1,'FontSize',14,'YScale','log','FontName','Times');
box(axes1,'on');
hold(axes1,'on');

% Create multiple lines using matrix input to plot
plot1 = plot(P_tx_all_dBm,[E_all1 ] ,'MarkerSize',10, 'LineWidth',1.5, 'Parent',axes1);
set(plot1(1),'DisplayName','Exact (Monte Carlo)','Marker','diamond','LineStyle','none',...
    'Color',[0 0 0]);
set(plot1(2),'DisplayName','Proposed','Marker','*','Color',[0 0 1]);
set(plot1(3),'DisplayName','L ($\eta_{\rm L} = 0.02$)','Marker','o','LineStyle','--',...
    'Color',[1 0.1 0]);
set(plot1(4),'DisplayName','CL ($\eta_{\rm CL} = 0.2$)','Marker','x',...
    'LineStyle',':', ...
    'Color',[0.1 0.8 0]);
set(plot1(5),'DisplayName','CLC ($\eta_{\rm CLC} = 0.2$)','Marker', 'square','LineStyle','-.',...
    'Color',[0.870588235294118 0.490196078431373 0]);
set(plot1(6),'DisplayName','L ($\eta_{\rm L} = 0.005$)','Marker','<','LineStyle','--',...
    'Color',[1 0.1 0] );
set(plot1(7),'DisplayName','CL ($\eta_{\rm CL} = 0.1$)','Marker','+',...
    'LineStyle',':' , ...
    'Color',[0.1 0.8 0]);
set(plot1(8),'DisplayName','CLC ($\eta_{\rm CLC} = 0.1$)','Marker','hexagram','LineStyle','-.',...
    'Color',[0.870588235294118 0.490196078431373 0]);


% Create xlabel
xlabel({'$P_{\rm T}$ (dBm)'},'FontSize',16,...
    'FontName','Times New Roman',...
    'Interpreter','latex');

% Create ylabel
ylabel({'Expected harvested energy (mJ)'},'FontSize',16,...
    'FontName','Times New Roman');


% Create legend
legend1 = legend(axes1,'show')... 'Exact (Monte Carlo)',  'Proposed', ...
%     'L', 'CL', 'CLC');
%  legend1 = legend(axes1);
set(legend1,...
    'Location','southwest',...
    'Interpreter','latex',...
    'FontSize',15,...
    'FontName','Times New Roman');

xlim([P_tx_all_dBm(1), P_tx_all_dBm(end)])