close all;
clear all;
clc;

%
format long;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load 'ws_efficiency_rectenna_MTT'
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




%power consumption  
P_c_th_all_dB = [-35:2:-15];
P_c_th_all = 10.^(P_c_th_all_dB/10);
beta = 1e-5;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%carrier frequency and propagation velocity
F_car = 868*10^6;
lambda_c =  (3*10^8)/ F_car;
%reference distance
d_ref = 1;
%transmit power
P_tx = 1500; %mWatt
%path loss exponent
nu = 2.1;
%nakagami parameter
M_nakagami = 5;
%distance
d_all = [3, 5]; %meters
%average received power
cons_Pin_all = P_tx * (lambda_c/(4*pi*d_ref))^2 * (d_ref./d_all).^nu;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%scattering efficiency
rho_u = 0.01;
%percentage of time the antenna load is Z_0
tau_harv = 0.5;
%portion of time for harvesting circuit
chi_harv = 0.5;
%total resources for harvesting
zeta_harv = tau_harv*chi_harv;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



%%  Noise Parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%reader bandwidth
BW_reader = 10^6;
%noise figure (dB)
noiseFigure = 4;
%noise energy (dBm/Hz)
N0_dB = -174 + noiseFigure;
%standard deviation of noise
sigma_u = sqrt(10^(N0_dB/10)*BW_reader);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% other constant efficiency values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%harvesting efficiency
eta_L_1 = 0.35;
eta_CL_1 = 0.35;
eta_CLC_1 = 0.35;

eta_L_2 = 0.25;
eta_CL_2 = 0.25;
eta_CLC_2 = 0.25;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% Proposed methododology
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%sensitivity and saturation of rectenna in MTT paper
sens_rect = 10^(sens_MTT_rectenna_dBm/10);
satur_rect = 10^(satur_MTT_rectenna_dBm/10);
% P_in axis for approximation in dB
P_in_test_dB = sens_MTT_rectenna_dBm : 0.05 : satur_MTT_rectenna_dBm ;

%P_in axis for approximation in mWatt
b_all = 10.^(P_in_test_dB/10);
%P_out function evaluated at b_all
v_all =  (P_harv_function( P_in_test_dB, sens_MTT_rectenna_dBm, satur_MTT_rectenna_dBm, w_opt ));
plot(b_all, v_all)
%slopes
l_all = (v_all(2:end) - v_all(1:end-1) ) ./ (b_all(2:end) - b_all(1:end-1) ) ;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% metrics
%%%%%%%%%%%%%%%%%%%%%%%
Event_Proposed = zeros(length(P_c_th_all), length(d_all));
Event_exact_MC = zeros(length(P_c_th_all), length(d_all));
Event_Linear1  =zeros(length(P_c_th_all), length(d_all));
Event_ConstantLinear1 = zeros(length(P_c_th_all), length(d_all));
Event_ConstantLinearConstant1 = zeros(length(P_c_th_all), length(d_all));
Event_Linear2  = zeros(length(P_c_th_all), length(d_all));
Event_ConstantLinear2 = zeros(length(P_c_th_all), length(d_all));
Event_ConstantLinearConstant2 = zeros(length(P_c_th_all), length(d_all));

% %% metrics
% %%%%%%%%%%%%%%%%%%%%%%%
Event_L1_MC  = zeros(length(P_c_th_all), length(d_all));
Event_CL1_MC = zeros(length(P_c_th_all), length(d_all));
Event_CLC1_MC = zeros(length(P_c_th_all), length(d_all));
 


%monte carlo iter
MC_MaxIter = 250000;

%% monte carlo phase
for P_c_iter = 1 : length(P_c_th_all)
    %current power consumption
    P_c_th = P_c_th_all(P_c_iter);
    
    
    for d_iter = 1 : length(d_all)
        
        %received power
        cons_Pin = cons_Pin_all(d_iter);
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %thresholds theta_1 and theta_2
        %theta_1
        theta_1 = R_inv( beta )*sigma_u*sqrt(P_tx/rho_u) ;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% proposed methodology
        %theta max for proposed model
        if P_c_th < v_all(end)
            theta_Max_proposed =  max(theta_1 ,  Approx_PieceWise_Linear_function_inv( P_c_th, v_all, b_all )/zeta_harv );
            
            %outage event for the proposed model
            Event_Proposed(P_c_iter, d_iter) = 1 - gamcdf(theta_Max_proposed , M_nakagami , cons_Pin / M_nakagami );
        else
            Event_Proposed(P_c_iter, d_iter) = 0;
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% linear model
        %1
        %theta max for linear model
        theta_Max_linear1 =  max(theta_1 , P_c_th/(zeta_harv*eta_L_1));
        %outage event for the linear model
        Event_Linear1(P_c_iter, d_iter)  = 1 -  gamcdf(theta_Max_linear1 , M_nakagami , cons_Pin / M_nakagami );
        %2
        %theta max for linear model
        theta_Max_linear2 =  max( theta_1 ,  P_c_th/(zeta_harv*eta_L_2));
        %outage event for the linear model
        Event_Linear2(P_c_iter, d_iter)  = 1 -  gamcdf(theta_Max_linear2 , M_nakagami , cons_Pin / M_nakagami );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        
         %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% constant-linear model
        %1
        %theta max for constant linear model
        theta_Max_constant_linear1 =  max(theta_1 , (P_c_th/(eta_CL_1) + sens_rect)/zeta_harv);
        %outage event for the constant-linear model
        Event_ConstantLinear1(P_c_iter, d_iter)  = 1 -  gamcdf(theta_Max_constant_linear1 , M_nakagami ,  cons_Pin / M_nakagami );
        %2
        %theta max for constant linear model
        theta_Max_constant_linear2 =  max(theta_1 , (P_c_th/(eta_CL_2)+ sens_rect)/zeta_harv);
        %outage event for the constant-linear model
        Event_ConstantLinear2(P_c_iter, d_iter)  = 1 -  gamcdf(theta_Max_constant_linear2 , M_nakagami ,  cons_Pin / M_nakagami );
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%% constant-linear-constant model
        %1
        %theta max for constant linear model
        %     p_CLC_theta_1_1 = (theta_1 >= sens_rect & theta_1 <= satur_rect)* eta_CLC_1*theta_1 +  (theta_1 > satur_rect)*eta_CLC_1*satur_rect;
        if P_c_th < eta_CLC_1*satur_rect
            theta_Max_constant_linear_consant1 =  max(theta_1 ,  (P_c_th/(eta_CLC_1) + sens_rect)/zeta_harv);
            %outage event for the constant-linear model
            Event_ConstantLinearConstant1(P_c_iter, d_iter)  =   (1 -  gamcdf(theta_Max_constant_linear_consant1 , M_nakagami , cons_Pin / M_nakagami ));
            %2
            %theta max for constant linear model
            theta_Max_constant_linear_consant2 =   max(theta_1 ,  (P_c_th/(eta_CLC_2) + sens_rect)/zeta_harv);
            %outage event for the constant-linear model
            Event_ConstantLinearConstant2(P_c_iter, d_iter)  =   (1 -  gamcdf(theta_Max_constant_linear_consant2 , M_nakagami , cons_Pin / M_nakagami ));
        else
            Event_ConstantLinearConstant1(P_c_iter, d_iter)  = 0;
            Event_ConstantLinearConstant2(P_c_iter, d_iter) = 0;
        end
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %parameters to generate gamma random variables
        gamma_param_1 = M_nakagami *  ones(1,MC_MaxIter);
        gamma_param_2 =cons_Pin./gamma_param_1;
        
        %generate gamma random variables with the specific parameters
        P_in =  gamrnd(gamma_param_1 ,gamma_param_2  );
        
        %total outage event for the proposed system
        P_out_exact_MC =  P_harv_function(10*log10(zeta_harv*P_in),sens_MTT_rectenna_dBm, satur_MTT_rectenna_dBm, w_opt);
        
        %event of outage for actual system
        Event_exact_MC(P_c_iter, d_iter) = mean(P_in >= theta_1 &  P_out_exact_MC >= P_c_th);
        
        
        % baseline models
        Event_L1_MC(P_c_iter, d_iter) =  mean(P_in >= theta_1 &  eta_L_1*zeta_harv*P_in >= P_c_th);
        Event_CL1_MC(P_c_iter, d_iter) =  mean(P_in >= theta_1 &  ((zeta_harv*P_in >= sens_rect) .* (eta_CL_1*zeta_harv*P_in)) >= P_c_th);
        Event_CLC1_MC(P_c_iter, d_iter) =  mean(P_in >= theta_1 & ...
            ((zeta_harv*P_in >= sens_rect & zeta_harv*P_in <= satur_rect).* ...
            (eta_CLC_1*zeta_harv*P_in) +  ((zeta_harv*P_in > satur_rect).*eta_CLC_1*satur_rect*zeta_harv) )>= P_c_th );
    end
    
end
Event_exact_MC(:,1)'
Event_Proposed(:,1)'
Event_Linear1(:,1)'
Event_L1_MC(:,1)'
 Event_ConstantLinear1(:,1)'
 Event_CL1_MC(:,1)'
 Event_ConstantLinearConstant1(:,1)'
 Event_CLC1_MC(:,1)'
%%
a1 = [ Event_exact_MC(:,1)' ; Event_Proposed(:,1)' ;Event_Linear1(:,1)'; Event_ConstantLinear1(:,1)'; Event_ConstantLinearConstant1(:,1)';...
    Event_Linear2(:,1)'; Event_ConstantLinear2(:,1)'; Event_ConstantLinearConstant2(:,1)']';
a2 = [ Event_exact_MC(:,2)' ; Event_Proposed(:,2)' ;Event_Linear1(:,2)'; Event_ConstantLinear1(:,2)'; Event_ConstantLinearConstant1(:,2)';...
    Event_Linear2(:,2)'; Event_ConstantLinear2(:,2)'; Event_ConstantLinearConstant2(:,2)']';

% % Create figure
figure1 = figure;
%
% Create axes
axes1 = axes('Parent',figure1,'FontSize',14,'FontName','Times');
box(axes1,'on');
hold(axes1,'on');

% Create multiple lines using matrix input to plot
plot1 = plot(P_c_th_all_dB,[a1 a2 ] ,'MarkerSize',10, 'LineWidth',1.5, 'Parent',axes1);
set(plot1(1),'DisplayName','Exact (Monte Carlo)','Marker','diamond','LineStyle','none',...
    'Color',[0 0 0]);
set(plot1(2),'DisplayName','Proposed','Marker','*','Color',[0 0 1]);
set(plot1(3),'DisplayName','Linear (L) (\eta_{\rm L} = 0.3)','Marker','o','LineStyle','--',...
    'Color',[1 0.1 0]);
set(plot1(4),'DisplayName','Constant-Linear (CL) ($\eta_{\rm CL} = 0.3$)','Marker','x',...
    'LineStyle',':', ...
    'Color',[0.1 0.8 0]);
set(plot1(5),'DisplayName','Linear (CLC) ($\eta_{CLC} = 0.3$)','Marker', 'square','LineStyle','-.',...
    'Color',[0.870588235294118 0.490196078431373 0]);
set(plot1(6),'DisplayName','Linear (L) ($\eta_L = 0.25$)','Marker','<','LineStyle','--',...
    'Color',[1 0.1 0] );
set(plot1(7),'DisplayName','Constant-Linear (CL) (\eta_{CL} = 0.25)','Marker','+',...
    'LineStyle',':' , ...
    'Color',[0.1 0.8 0]);
set(plot1(8),'DisplayName','Linear (CLC) (\eta_{CLC} = 0.3)','Marker','hexagram','LineStyle','-.',...
    'Color',[0.870588235294118 0.490196078431373 0]);
set(plot1(9),'DisplayName','Exact','Marker','diamond','LineStyle','none',...
    'Color',[0 0 0]);
set(plot1(10),'DisplayName','Proposed','Marker','*','Color',[0 0 1]);
set(plot1(11),'DisplayName','Linear (L) (\eta_L = 0.3)','Marker','o','LineStyle','--',...
    'Color',[1 0.1 0] );
set(plot1(12),'DisplayName','Constant-Linear (CL) (\eta_{CL} = 0.3)','Marker', 'x' ,...
    'LineStyle',':' , ...
    'Color',[0.1 0.8 0]);
set(plot1(13),'DisplayName','Linear (CLC) (\eta_{CLC} = 0.3)','Marker','square','LineStyle','-.',...
    'Color',[0.870588235294118 0.490196078431373 0]);
set(plot1(14),'DisplayName','Linear (L) (\eta_L = 0.25)','Marker','<','LineStyle','--',...
    'Color',[1 0.1 0] );
set(plot1(15),'DisplayName','Constant-Linear (CL) (\eta_{CL} = 0.25)','Marker','+',...
    'LineStyle',':', ...
    'Color',[0.1 0.8 0] );
set(plot1(16),'DisplayName','Linear (CLC) (\eta_{CLC} = 0.3)','Marker','hexagram','LineStyle','-.',...
    'Color',[0.870588235294118 0.490196078431373 0]);



% Create xlabel
xlabel({'$\mathtt{P}_{\rm c}$ (dBm)'},'FontSize',18,...
    'FontName','Times New Roman',...
    'Interpreter','latex');

% Create ylabel
ylabel({'Successful reception at interrogator'},'FontSize',18,...
    'FontName','Times New Roman');


% Create legend
legend1 = legend(axes1,'show', 'Exact (Monte Carlo)',  'Proposed', ...
    'L ($\eta_{\rm L} = 0.35$)', 'CL ($\eta_{\rm CL} = 0.35$)',...
    'CLC ($\eta_{\rm CLC} = 0.35$)', 'L  ($\eta_{\rm L} = 0.25$)', ...
    'CL ($\eta_{\rm CL} = 0.25$)',...
    'CLC  ($\eta_{\rm CLC} = 0.25$)');
set(legend1,...
    'Location','southwest',...
    'Interpreter','latex',...
    'FontSize',16,...
    'FontName','Times New Roman');